/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | Copyright (C) 1991-2005 OpenCFD Ltd.
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
    Free Software Foundation; either version 2 of the License, or (at your
    option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM; if not, write to the Free Software Foundation,
    Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

\*---------------------------------------------------------------------------*/

#include "heatFluxFvPatchScalarField.H"
#include "addToRunTimeSelectionTable.H"
#include "volFields.H"
#include "surfaceFields.H"
#include "fvcMeshPhi.H"
#include "turbulenceModel.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

namespace Foam
{

// * * * * * * * * * * * * * * * * Constructors  * * * * * * * * * * * * * * //

heatFluxFvPatchScalarField::heatFluxFvPatchScalarField
(
    const fvPatch& p,
    const scalarField& iF
)
:
  fixedGradientFvPatchScalarField(p, iF),
  hf_(p.size(),0)
{
}


heatFluxFvPatchScalarField::heatFluxFvPatchScalarField
(
    const heatFluxFvPatchScalarField& ptf,
    const fvPatch& p,
    const scalarField& iF,
    const fvPatchFieldMapper& mapper
)
:
  fixedGradientFvPatchScalarField(ptf, p, iF, mapper),
  hf_(ptf.hf_,mapper)
{  
}


heatFluxFvPatchScalarField::heatFluxFvPatchScalarField
(
    const fvPatch& p,
    const scalarField& iF,
    const dictionary& dict
)
:
  fixedGradientFvPatchScalarField(p, iF),
  hf_("heatFluxPerSquareMeter",dict,p.size())
{
  gradient()=scalarField("gradient", dict, p.size());
  fixedGradientFvPatchScalarField::updateCoeffs();
  fixedGradientFvPatchScalarField::evaluate();
}


heatFluxFvPatchScalarField::heatFluxFvPatchScalarField
(
    const heatFluxFvPatchScalarField& pivpvf,
    const scalarField& iF
)
:
  fixedGradientFvPatchScalarField(pivpvf, iF),
  hf_(pivpvf.hf_)
{
}


// * * * * * * * * * * * * * * * Member Functions  * * * * * * * * * * * * * //


void heatFluxFvPatchScalarField::autoMap
(
    const fvPatchFieldMapper& m
)
{
    scalarField::autoMap(m);
    hf_.autoMap(m);
}


void heatFluxFvPatchScalarField::rmap
(
    const fvPatchScalarField& ptf,
    const labelList& addr
)
{
    fixedGradientFvPatchScalarField::rmap(ptf, addr);

    const heatFluxFvPatchScalarField& tiptf =
        refCast<const heatFluxFvPatchScalarField>(ptf);

    hf_.rmap(tiptf.hf_, addr);
}

// Update the coefficients associated with the patch field
void heatFluxFvPatchScalarField::updateCoeffs()
{
    if (updated())
    {
        return;
    }

    const compressible::turbulenceModel& rasModel =
      db().lookupObject<compressible::turbulenceModel>("turbulenceProperties");

    scalarField alphaEffWall =
      rasModel.alphaEff()().boundaryField()[patch().patch().index()];

    gradient() = hf_/alphaEffWall;

    fixedGradientFvPatchScalarField::updateCoeffs();
}


// Write
void heatFluxFvPatchScalarField::write(Ostream& os) const
{
    fixedGradientFvPatchScalarField::write(os);
    writeEntry("value", os);
    hf_.writeEntry("heatFluxPerSquareMeter",os);
}


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

makePatchTypeField(fvPatchScalarField, heatFluxFvPatchScalarField);

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

} // End namespace Foam

// ************************************************************************* //
